{% load bleach_tags %}
{% load i18n %}

{% for agreement in form.agreements %}
    {% if agreement.obligatory %}
        <p>
            {{ agreement.text | bleach }}
        </p>
    {% else %}
        <div>
            <input type="checkbox" name="agreement_{{ agreement.id }}"
                {% if agreement.agreed %}checked{% endif %}
            >
            {{ agreement.text | bleach }}
        </div>
    {% endif %}
{% endfor %}