{% load i18n %}

<html>
    <meta charset="UTF-8">
    <body>
        <h1>{% trans 'To continue, you have to accept the following terms:'%}</h1>
        <form method="post">
            {% csrf_token %}
            {% include 'django-open-id/consent-form.html' %}
            <br/>
            <button type="submit" name="agree">{% blocktrans with client_name=client.client_name %}I accept the terms and want to continue to {{ client_name }}{% endblocktrans %}</button>
        </form>
    </body>
</html>